@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2009 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2018 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -14,7 +14,7 @@
  *
  *    You should have received a copy of the GNU General Public License
  *    along with this program; if not, write to the Free Software
- *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 # include "config.h"
@@ -24,6 +24,7 @@
 # include  <iostream>
 
 # include  "netlist.h"
+# include  "netvector.h"
 # include  "netmisc.h"
 # include  "ivl_assert.h"
 
@@ -39,8 +40,7 @@ static NetNet* convert_to_real_const(Design*des, NetScope*scope, NetEConst*expr)
   /* Note that lsig, rsig and real_args are references. */
 static bool process_binary_args(Design*des, NetScope*scope, NetExpr*root,
 				NetExpr*left, NetExpr*right,
-				NetNet*&lsig, NetNet*&rsig, bool&real_args,
-				NetExpr*obj)
+				NetNet*&lsig, NetNet*&rsig, bool&real_args)
 {
       if (left->expr_type() == IVL_VT_REAL ||
           right->expr_type() == IVL_VT_REAL) {
@@ -77,7 +77,7 @@ static bool process_binary_args(Design*des, NetScope*scope, NetExpr*root,
       else return false;
 }
 
-NetNet* NetExpr::synthesize(Design*des, NetScope*scope, NetExpr*root)
+NetNet* NetExpr::synthesize(Design*des, NetScope*, NetExpr*)
 {
       cerr << get_fileline() << ": internal error: cannot synthesize expression: "
 	   << *this << endl;
@@ -95,7 +95,7 @@ NetNet* NetEBAdd::synthesize(Design*des, NetScope*scope, NetExpr*root)
       NetNet *lsig=0, *rsig=0;
       bool real_args=false;
       if (process_binary_args(des, scope, root, left_, right_, lsig, rsig,
-                              real_args, this)) {
+                              real_args)) {
 	    return 0;
       }
 
@@ -119,12 +119,15 @@ NetNet* NetEBAdd::synthesize(Design*des, NetScope*scope, NetExpr*root)
       }
 
       perm_string path = lsig->scope()->local_symbol();
-      NetNet*osig = new NetNet(lsig->scope(), path, NetNet::IMPLICIT, width);
+      netvector_t*osig_vec = new netvector_t(expr_type(), width-1, 0);
+      osig_vec->set_signed(has_sign());
+      NetNet*osig = new NetNet(lsig->scope(), path, NetNet::IMPLICIT, osig_vec);
+      osig->set_line(*this);
       osig->local_flag(true);
-      osig->data_type(expr_type());
 
       perm_string oname = osig->scope()->local_symbol();
       NetAddSub *adder = new NetAddSub(lsig->scope(), oname, width);
+      adder->set_line(*this);
       connect(lsig->pin(0), adder->pin_DataA());
       connect(rsig->pin(0), adder->pin_DataB());
       connect(osig->pin(0), adder->pin_Result());
@@ -171,34 +174,36 @@ NetNet* NetEBBits::synthesize(Design*des, NetScope*scope, NetExpr*root)
       rsig = pad_to_width(des, rsig, width, *this);
 
       assert(lsig->vector_width() == rsig->vector_width());
+      netvector_t*osig_vec = new netvector_t(expr_type(), width-1, 0);
       NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::IMPLICIT, width);
+			       NetNet::IMPLICIT, osig_vec);
+      osig->set_line(*this);
       osig->local_flag(true);
-      osig->data_type(expr_type());
 
       perm_string oname = scope->local_symbol();
       NetLogic*gate;
 
       switch (op()) {
 	  case '&':
-	    gate = new NetLogic(scope, oname, 3, NetLogic::AND, width);
+	    gate = new NetLogic(scope, oname, 3, NetLogic::AND, width, true);
 	    break;
 	  case 'A':
-	    gate = new NetLogic(scope, oname, 3, NetLogic::NAND, width);
+	    gate = new NetLogic(scope, oname, 3, NetLogic::NAND, width, true);
 	    break;
 	  case '|':
-	    gate = new NetLogic(scope, oname, 3, NetLogic::OR, width);
+	    gate = new NetLogic(scope, oname, 3, NetLogic::OR, width, true);
 	    break;
 	  case '^':
-	    gate = new NetLogic(scope, oname, 3, NetLogic::XOR, width);
+	    gate = new NetLogic(scope, oname, 3, NetLogic::XOR, width, true);
 	    break;
 	  case 'O':
-	    gate = new NetLogic(scope, oname, 3, NetLogic::NOR, width);
+	    gate = new NetLogic(scope, oname, 3, NetLogic::NOR, width, true);
 	    break;
 	  case 'X':
-	    gate = new NetLogic(scope, oname, 3, NetLogic::XNOR, width);
+	    gate = new NetLogic(scope, oname, 3, NetLogic::XNOR, width, true);
 	    break;
 	  default:
+	    gate = NULL;
 	    assert(0);
       }
 
@@ -219,7 +224,7 @@ NetNet* NetEBComp::synthesize(Design*des, NetScope*scope, NetExpr*root)
       unsigned width;
       bool real_args=false;
       if (process_binary_args(des, scope, root, left_, right_, lsig, rsig,
-                              real_args, this)) {
+                              real_args)) {
 	    return 0;
       }
 
@@ -239,11 +244,11 @@ NetNet* NetEBComp::synthesize(Design*des, NetScope*scope, NetExpr*root)
 		  rsig = pad_to_width(des, rsig, width, *this);
       }
 
+      netvector_t*osig_vec = new netvector_t(IVL_VT_LOGIC);
       NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::IMPLICIT, 1);
+			       NetNet::IMPLICIT, osig_vec);
       osig->set_line(*this);
       osig->local_flag(true);
-      osig->data_type(IVL_VT_LOGIC);
 
 	// Test if the comparison is signed.
 	//
@@ -269,7 +274,7 @@ NetNet* NetEBComp::synthesize(Design*des, NetScope*scope, NetExpr*root)
 
       if (op_ == 'E' || op_ == 'N') {
 	    NetCaseCmp*gate = new NetCaseCmp(scope, scope->local_symbol(),
-					     width, op_=='E'?true:false);
+					     width, op_=='E'?NetCaseCmp::EEQ:NetCaseCmp::NEQ);
 	    gate->set_line(*this);
 	    connect(gate->pin(0), osig->pin(0));
 	    connect(gate->pin(1), lsig->pin(0));
@@ -282,7 +287,7 @@ NetNet* NetEBComp::synthesize(Design*des, NetScope*scope, NetExpr*root)
 	   operation. Make an XNOR gate instead of a comparator. */
       if ((width == 1) && (op_ == 'e') && !real_args) {
 	    NetLogic*gate = new NetLogic(scope, scope->local_symbol(),
-					 3, NetLogic::XNOR, 1);
+					 3, NetLogic::XNOR, 1, true);
 	    gate->set_line(*this);
 	    connect(gate->pin(0), osig->pin(0));
 	    connect(gate->pin(1), lsig->pin(0));
@@ -296,7 +301,7 @@ NetNet* NetEBComp::synthesize(Design*des, NetScope*scope, NetExpr*root)
 	   an XOR instead of an XNOR gate. */
       if ((width == 1) && (op_ == 'n')  && !real_args) {
 	    NetLogic*gate = new NetLogic(scope, scope->local_symbol(),
-					 3, NetLogic::XOR, 1);
+					 3, NetLogic::XOR, 1, true);
 	    gate->set_line(*this);
 	    connect(gate->pin(0), osig->pin(0));
 	    connect(gate->pin(1), lsig->pin(0));
@@ -330,6 +335,7 @@ NetNet* NetEBComp::synthesize(Design*des, NetScope*scope, NetExpr*root)
 		  des->errors += 1;
 		  return 0;
 	    }
+	    // fallthrough
 	  case 'e': // ==
 	    connect(dev->pin_AEB(), osig->pin(0));
 	    break;
@@ -348,6 +354,7 @@ NetNet* NetEBComp::synthesize(Design*des, NetScope*scope, NetExpr*root)
 		  des->errors += 1;
 		  return 0;
 	    }
+	    // fallthrough
 	  case 'n': // !=
 	    connect(dev->pin_ANEB(), osig->pin(0));
 	    break;
@@ -368,7 +375,7 @@ NetNet* NetEBPow::synthesize(Design*des, NetScope*scope, NetExpr*root)
       unsigned width;
       bool real_args=false;
       if (process_binary_args(des, scope, root, left_, right_, lsig, rsig,
-                              real_args, this)) {
+                              real_args)) {
 	    return 0;
       }
 
@@ -378,18 +385,20 @@ NetNet* NetEBPow::synthesize(Design*des, NetScope*scope, NetExpr*root)
       NetPow*powr = new NetPow(scope, scope->local_symbol(), width,
 			       lsig->vector_width(),
 			       rsig->vector_width());
+      powr->set_line(*this);
       des->add_node(powr);
 
-      powr->set_signed( has_sign() );
-      powr->set_line(*this);
+        // The lpm_pwr object only cares about the signedness of the exponent.
+      powr->set_signed( right_->has_sign() );
 
       connect(powr->pin_DataA(), lsig->pin(0));
       connect(powr->pin_DataB(), rsig->pin(0));
 
+      netvector_t*osig_vec = new netvector_t(expr_type(), width-1, 0);
+      osig_vec->set_signed(has_sign());
       NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::IMPLICIT, width);
+			       NetNet::IMPLICIT, osig_vec);
       osig->set_line(*this);
-      osig->data_type(expr_type());
       osig->local_flag(true);
 
       connect(powr->pin_Result(), osig->pin(0));
@@ -403,7 +412,7 @@ NetNet* NetEBMult::synthesize(Design*des, NetScope*scope, NetExpr*root)
       unsigned width;
       bool real_args=false;
       if (process_binary_args(des, scope, root, left_, right_, lsig, rsig,
-                              real_args, this)) {
+                              real_args)) {
 	    return 0;
       }
 
@@ -414,18 +423,19 @@ NetNet* NetEBMult::synthesize(Design*des, NetScope*scope, NetExpr*root)
 				 width,
 				 lsig->vector_width(),
 				 rsig->vector_width());
+      mult->set_line(*this);
       des->add_node(mult);
 
       mult->set_signed( has_sign() );
-      mult->set_line(*this);
 
       connect(mult->pin_DataA(), lsig->pin(0));
       connect(mult->pin_DataB(), rsig->pin(0));
 
+      netvector_t*osig_vec = new netvector_t(expr_type(), width-1, 0);
+      osig_vec->set_signed(has_sign());
       NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::IMPLICIT, width);
+			       NetNet::IMPLICIT, osig_vec);
       osig->set_line(*this);
-      osig->data_type(expr_type());
       osig->local_flag(true);
 
       connect(mult->pin_Result(), osig->pin(0));
@@ -439,18 +449,18 @@ NetNet* NetEBDiv::synthesize(Design*des, NetScope*scope, NetExpr*root)
       unsigned width;
       bool real_args=false;
       if (process_binary_args(des, scope, root, left_, right_, lsig, rsig,
-                              real_args, this)) {
+                              real_args)) {
 	    return 0;
       }
 
       if (real_args) width = 1;
       else width = expr_width();
 
+      netvector_t*osig_vec = new netvector_t(lsig->data_type(), width-1, 0);
+      osig_vec->set_signed(has_sign());
       NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::IMPLICIT, width);
+			       NetNet::IMPLICIT, osig_vec);
       osig->set_line(*this);
-      osig->data_type(lsig->data_type());
-      osig->set_signed(has_sign());
       osig->local_flag(true);
 
       switch (op()) {
@@ -514,72 +524,46 @@ NetNet* NetEBLogic::synthesize(Design*des, NetScope*scope, NetExpr*root)
 
       if (lsig == 0 || rsig == 0) return 0;
 
-        /* You cannot currently do logical operations on real values. */
+        /* Any real value should have already been converted to a bit value. */
       if (lsig->data_type() == IVL_VT_REAL ||
           rsig->data_type() == IVL_VT_REAL) {
-	    cerr << get_fileline() << ": sorry: " << human_readable_op(op_)
-	         << " is currently unsupported for real values." << endl;
+	    cerr << get_fileline() << ": internal error: "
+	         << human_readable_op(op_)
+	         << " is missing real to bit conversion." << endl;
 	    des->errors += 1;
 	    return 0;
       }
 
+      netvector_t*osig_tmp = new netvector_t(expr_type());
       NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::IMPLICIT, 1);
-      osig->data_type(expr_type());
+			       NetNet::IMPLICIT, osig_tmp);
+      osig->set_line(*this);
       osig->local_flag(true);
 
+      NetLogic*olog;
+      perm_string oname = scope->local_symbol();
 
+	/* Create the logic OR/AND gate. This has a single bit output,
+	 * with single bit inputs for the two operands. */
       if (op() == 'o') {
-
-	      /* Logic OR can handle the reduction *and* the logical
-		 comparison with a single wide OR gate. So handle this
-		 magically. */
-
-	    perm_string oname = scope->local_symbol();
-
-	    NetLogic*olog = new NetLogic(scope, oname,
-					 lsig->pin_count()+rsig->pin_count()+1,
-					 NetLogic::OR, 1);
-
-	    connect(osig->pin(0), olog->pin(0));
-
-	    unsigned pin = 1;
-	    for (unsigned idx = 0 ;  idx < lsig->pin_count() ;  idx = 1)
-		  connect(olog->pin(pin+idx), lsig->pin(idx));
-
-	    pin += lsig->pin_count();
-	    for (unsigned idx = 0 ;  idx < rsig->pin_count() ;  idx = 1)
-		  connect(olog->pin(pin+idx), rsig->pin(idx));
-
-	    des->add_node(olog);
-
+	    olog = new NetLogic(scope, oname, 3, NetLogic::OR, 1, true);
       } else {
 	    assert(op() == 'a');
-
-	      /* Create the logic AND gate. This is a single bit
-		 output, with inputs for each of the operands. */
-	    NetLogic*olog;
-	    perm_string oname = scope->local_symbol();
-
-	    olog = new NetLogic(scope, oname, 3, NetLogic::AND, 1);
-
-	    connect(osig->pin(0), olog->pin(0));
-	    des->add_node(olog);
-
-	      /* XXXX Here, I need to reduce the parameters with
-		 reduction or. */
-
-
-	      /* By this point, the left and right parameters have been
-		 reduced to single bit values. Now we just connect them to
-		 the logic gate. */
-	    assert(lsig->pin_count() == 1);
-	    connect(lsig->pin(0), olog->pin(1));
-
-	    assert(rsig->pin_count() == 1);
-	    connect(rsig->pin(0), olog->pin(2));
+	    olog = new NetLogic(scope, oname, 3, NetLogic::AND, 1, true);
       }
 
+      olog->set_line(*this);
+      des->add_node(olog);
+
+      connect(osig->pin(0), olog->pin(0));
+
+	/* The left and right operands have already been reduced to a
+	 * single bit value, so just connect then to the logic gate. */
+      assert(lsig->pin_count() == 1);
+      connect(lsig->pin(0), olog->pin(1));
+
+      assert(rsig->pin_count() == 1);
+      connect(rsig->pin(0), olog->pin(2));
 
       return osig;
 }
@@ -601,8 +585,8 @@ NetNet* NetEBShift::synthesize(Design*des, NetScope*scope, NetExpr*root)
 	    return 0;
       }
 
-      const bool right_flag  =  op_ == 'r' || op_ == 'R';
-      const bool signed_flag =  op_ == 'R';
+      const bool right_flag  = op_ == 'r' || op_ == 'R';
+      const bool signed_flag = has_sign() && op_ == 'R';
 
 	/* Detect the special case where the shift amount is
 	   constant. Evaluate the shift amount, and simply reconnect
@@ -617,9 +601,10 @@ NetNet* NetEBShift::synthesize(Design*des, NetScope*scope, NetExpr*root)
 	    if (shift == 0)
 		  return lsig;
 
+	    netvector_t*osig_vec = new netvector_t(expr_type(), expr_width()-1,0);
 	    NetNet*osig = new NetNet(scope, scope->local_symbol(),
-				     NetNet::IMPLICIT, expr_width());
-	    osig->data_type(expr_type());
+				     NetNet::IMPLICIT, osig_vec);
+	    osig->set_line(*this);
 	    osig->local_flag(true);
 
 	      // ushift is the amount of pad created by the shift.
@@ -633,14 +618,16 @@ NetNet* NetEBShift::synthesize(Design*des, NetScope*scope, NetExpr*root)
 	      // to the amount left by the shift.
 	    NetPartSelect*psel = new NetPartSelect(lsig, shift<0? ushift : 0,
 						   part_width,
-						   NetPartSelect::VP);
+						   NetPartSelect::VP,
+	                                           signed_flag && right_flag);
+	    psel->set_line(*this);
 	    des->add_node(psel);
 
+	    netvector_t*psig_vec = new netvector_t(expr_type(), part_width-1, 0);
 	    NetNet*psig = new NetNet(scope, scope->local_symbol(),
-				     NetNet::IMPLICIT, part_width);
-	    psig->data_type(expr_type());
-	    psig->local_flag(true);
+				     NetNet::IMPLICIT, psig_vec);
 	    psig->set_line(*this);
+	    psig->local_flag(true);
 	    connect(psig->pin(0), psel->pin(0));
 
 	      // Handle the special case of a signed right shift. In
@@ -649,8 +636,8 @@ NetNet* NetEBShift::synthesize(Design*des, NetScope*scope, NetExpr*root)
 	    if (signed_flag && right_flag) {
 		  NetSignExtend*pad = new NetSignExtend(scope, scope->local_symbol(),
 							osig->vector_width());
-		  des->add_node(pad);
 		  pad->set_line(*this);
+		  des->add_node(pad);
 
 		  connect(pad->pin(1), psig->pin(0));
 		  connect(pad->pin(0), osig->pin(0));
@@ -664,11 +651,12 @@ NetNet* NetEBShift::synthesize(Design*des, NetScope*scope, NetExpr*root)
 					 znum);
 	    des->add_node(zcon);
 
+	    netvector_t*zsig_vec = new netvector_t(osig->data_type(),
+						   znum.len()-1, 0);
 	    NetNet*zsig = new NetNet(scope, scope->local_symbol(),
-				     NetNet::WIRE, znum.len());
-	    zsig->data_type(osig->data_type());
-	    zsig->local_flag(true);
+				     NetNet::WIRE, zsig_vec);
 	    zsig->set_line(*this);
+	    zsig->local_flag(true);
 	    connect(zcon->pin(0), zsig->pin(0));
 
 	    NetConcat*ccat = new NetConcat(scope, scope->local_symbol(),
@@ -694,9 +682,10 @@ NetNet* NetEBShift::synthesize(Design*des, NetScope*scope, NetExpr*root)
 
       if (rsig == 0) return 0;
 
+      netvector_t*osig_vec = new netvector_t(expr_type(), expr_width()-1, 0);
       NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::IMPLICIT, expr_width());
-      osig->data_type(expr_type());
+			       NetNet::IMPLICIT, osig_vec);
+      osig->set_line(*this);
       osig->local_flag(true);
 
       NetCLShift*dev = new NetCLShift(scope, scope->local_symbol(),
@@ -719,32 +708,51 @@ NetNet* NetEBShift::synthesize(Design*des, NetScope*scope, NetExpr*root)
 NetNet* NetEConcat::synthesize(Design*des, NetScope*scope, NetExpr*root)
 {
 	/* First, synthesize the operands. */
-      unsigned nparms = parms_.count();
-      NetNet**tmp = new NetNet*[parms_.count()];
+      unsigned num_parms = parms_.size();
+      NetNet**tmp = new NetNet*[parms_.size()];
       bool flag = true;
-      for (unsigned idx = 0 ;  idx < parms_.count() ;  idx += 1) {
+      ivl_variable_type_t data_type = IVL_VT_NO_TYPE;
+      for (unsigned idx = 0 ;  idx < parms_.size() ;  idx += 1) {
 	    if (parms_[idx]->expr_width() == 0) {
-		  tmp[idx] = 0;
-		  nparms -= 1;
+		    /* We need to synthesize a replication of zero. */
+		  tmp[idx] = parms_[idx]->synthesize(des, scope, root);
+		  assert(tmp[idx] == 0);
+		  num_parms -= 1;
 	    } else {
 		  tmp[idx] = parms_[idx]->synthesize(des, scope, root);
 		  if (tmp[idx] == 0) flag = false;
+		    /* Set the data type to the first one found. */
+		  switch (data_type) {
+		      case IVL_VT_NO_TYPE:
+			data_type = tmp[idx]->data_type();
+			break;
+		      case IVL_VT_BOOL:
+			if (tmp[idx]->data_type()==IVL_VT_LOGIC)
+			      data_type = IVL_VT_LOGIC;
+			break;
+		      default:
+			break;
+		  }
 	    }
       }
 
       if (flag == false) return 0;
 
-      ivl_assert(*this, tmp[0]);
+      ivl_assert(*this, data_type != IVL_VT_NO_TYPE);
+
+	/* If this is a replication of zero just return 0. */
+      if (expr_width() == 0) return 0;
 
 	/* Make a NetNet object to carry the output vector. */
       perm_string path = scope->local_symbol();
-      NetNet*osig = new NetNet(scope, path, NetNet::IMPLICIT, expr_width());
+      netvector_t*osig_vec = new netvector_t(data_type, expr_width()-1, 0);
+      NetNet*osig = new NetNet(scope, path, NetNet::IMPLICIT, osig_vec);
+      osig->set_line(*this);
       osig->local_flag(true);
-      osig->data_type(tmp[0]->data_type());
 
       NetConcat*concat = new NetConcat(scope, scope->local_symbol(),
 				       osig->vector_width(),
-				       nparms * repeat());
+				       num_parms * repeat());
       concat->set_line(*this);
       des->add_node(concat);
       connect(concat->pin(0), osig->pin(0));
@@ -752,8 +760,8 @@ NetNet* NetEConcat::synthesize(Design*des, NetScope*scope, NetExpr*root)
       unsigned count_input_width = 0;
       unsigned cur_pin = 1;
       for (unsigned rpt = 0; rpt < repeat(); rpt += 1) {
-	    for (unsigned idx = 0 ;  idx < parms_.count() ;  idx += 1) {
-		  unsigned concat_item = parms_.count()-idx-1;
+	    for (unsigned idx = 0 ;  idx < parms_.size() ;  idx += 1) {
+		  unsigned concat_item = parms_.size()-idx-1;
 		  if (tmp[concat_item] == 0) continue;
 		  connect(concat->pin(cur_pin), tmp[concat_item]->pin(0));
 		  cur_pin += 1;
@@ -783,15 +791,15 @@ NetNet* NetEConst::synthesize(Design*des, NetScope*scope, NetExpr*)
 	    return 0;
       }
 
-      NetNet*osig = new NetNet(scope, path, NetNet::IMPLICIT, width);
-      osig->local_flag(true);
-      osig->data_type(expr_type());
-      osig->set_signed(has_sign());
+      netvector_t*osig_vec = new netvector_t(expr_type(), width-1, 0);
+      osig_vec->set_signed(has_sign());
+      NetNet*osig = new NetNet(scope, path, NetNet::IMPLICIT, osig_vec);
       osig->set_line(*this);
+      osig->local_flag(true);
 
       NetConst*con = new NetConst(scope, scope->local_symbol(), value());
-      des->add_node(con);
       con->set_line(*this);
+      des->add_node(con);
 
       connect(osig->pin(0), con->pin(0));
       return osig;
@@ -804,15 +812,15 @@ NetNet* NetECReal::synthesize(Design*des, NetScope*scope, NetExpr*)
 {
       perm_string path = scope->local_symbol();
 
-      NetNet*osig = new NetNet(scope, path, NetNet::WIRE, 1);
-      osig->local_flag(true);
-      osig->data_type(IVL_VT_REAL);
-      osig->set_signed(has_sign());
+      netvector_t*osig_vec = new netvector_t(IVL_VT_REAL);
+      osig_vec->set_signed(has_sign());
+      NetNet*osig = new NetNet(scope, path, NetNet::WIRE, osig_vec);
       osig->set_line(*this);
+      osig->local_flag(true);
 
       NetLiteral*con = new NetLiteral(scope, scope->local_symbol(), value_);
-      des->add_node(con);
       con->set_line(*this);
+      des->add_node(con);
 
       connect(osig->pin(0), con->pin(0));
       return osig;
@@ -837,9 +845,10 @@ NetNet* NetEUBits::synthesize(Design*des, NetScope*scope, NetExpr*root)
       }
 
       unsigned width = isig->vector_width();
+      netvector_t*osig_vec = new netvector_t(expr_type(), width-1, 0);
       NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::IMPLICIT, width);
-      osig->data_type(expr_type());
+			       NetNet::IMPLICIT, osig_vec);
+      osig->set_line(*this);
       osig->local_flag(true);
 
       perm_string oname = scope->local_symbol();
@@ -847,9 +856,11 @@ NetNet* NetEUBits::synthesize(Design*des, NetScope*scope, NetExpr*root)
 
       switch (op()) {
 	  case '~':
-	    gate = new NetLogic(scope, oname, 2, NetLogic::NOT, width);
+	    gate = new NetLogic(scope, oname, 2, NetLogic::NOT, width, true);
+	    gate->set_line(*this);
 	    break;
 	  default:
+	    gate = NULL;
 	    assert(0);
       }
 
@@ -877,22 +888,23 @@ NetNet* NetEUnary::synthesize(Design*des, NetScope*scope, NetExpr*root)
 	    if (expr_->has_sign() == false)
 		  return sub;
 
+	    netvector_t*sig_vec = new netvector_t(sub->data_type(),
+						  sub->vector_width()-1, 0);
 	    NetNet*sig = new NetNet(scope, scope->local_symbol(),
-				    NetNet::WIRE, sub->vector_width());
+				    NetNet::WIRE, sig_vec);
 	    sig->set_line(*this);
 	    sig->local_flag(true);
-	    sig->data_type(sub->data_type());
 
 	    NetAbs*tmp = new NetAbs(scope, scope->local_symbol(), sub->vector_width());
-	    des->add_node(tmp);
 	    tmp->set_line(*this);
+	    des->add_node(tmp);
 
 	    connect(tmp->pin(1), sub->pin(0));
 	    connect(tmp->pin(0), sig->pin(0));
 	    return sig;
       }
 
-      cerr << get_fileline() << ": iternal error: "
+      cerr << get_fileline() << ": internal error: "
 	   << "NetEUnary::synthesize cannot handle op_=" << op_ << endl;
       des->errors += 1;
       return expr_->synthesize(des, scope, root);
@@ -918,11 +930,6 @@ NetNet* NetEUReduce::synthesize(Design*des, NetScope*scope, NetExpr*root)
 	    return 0;
       }
 
-      NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::IMPLICIT, 1);
-      osig->data_type(expr_type());
-      osig->local_flag(true);
-
       NetUReduce::TYPE rtype = NetUReduce::NONE;
 
       switch (op()) {
@@ -953,8 +960,15 @@ NetNet* NetEUReduce::synthesize(Design*des, NetScope*scope, NetExpr*root)
 
       NetUReduce*gate = new NetUReduce(scope, scope->local_symbol(),
 				       rtype, isig->vector_width());
-
+      gate->set_line(*this);
       des->add_node(gate);
+
+      netvector_t*osig_vec = new netvector_t(expr_type());
+      NetNet*osig = new NetNet(scope, scope->local_symbol(),
+			       NetNet::IMPLICIT, osig_vec);
+      osig->set_line(*this);
+      osig->local_flag(true);
+
       connect(gate->pin(0), osig->pin(0));
       for (unsigned idx = 0 ;  idx < isig->pin_count() ;  idx += 1)
 	    connect(gate->pin(1+idx), isig->pin(idx));
@@ -962,6 +976,31 @@ NetNet* NetEUReduce::synthesize(Design*des, NetScope*scope, NetExpr*root)
       return osig;
 }
 
+NetNet* NetECast::synthesize(Design*des, NetScope*scope, NetExpr*root)
+{
+      NetNet*isig = expr_->synthesize(des, scope, root);
+
+      if (isig == 0) return 0;
+
+      switch (op()) {
+	  case 'v':
+	    isig = cast_to_int4(des, scope, isig, expr_width());
+	    break;
+	  case '2':
+	    isig = cast_to_int2(des, scope, isig, expr_width());
+	    break;
+	  case 'r':
+	    isig = cast_to_real(des, scope, isig);
+	    break;
+	  default:
+	    cerr << get_fileline() << ": internal error: "
+		 << "Unable to synthesize " << *this << "." << endl;
+	    return 0;
+      }
+
+      return isig;
+}
+
 /*
  * Turn a part/bit select expression into gates.
  * We know some things about the expression that elaboration enforces
@@ -977,8 +1016,6 @@ NetNet* NetESelect::synthesize(Design *des, NetScope*scope, NetExpr*root)
 
       if (sub == 0) return 0;
 
-      NetNet*off = 0;
-
 	// Detect the special case that there is a base expression and
 	// it is constant. In this case we can generate fixed part selects.
       if (NetEConst*base_const = dynamic_cast<NetEConst*>(base_)) {
@@ -993,11 +1030,12 @@ NetNet* NetESelect::synthesize(Design *des, NetScope*scope, NetExpr*root)
 	    }
 
 	    long base_val = base_tmp.as_long();
+	    unsigned below_width = 0;
 
 	      // Any below X bits?
 	    NetNet*below = 0;
 	    if (base_val < 0) {
-		  unsigned below_width = abs(base_val);
+		  below_width = abs(base_val);
 		  base_val = 0;
 		  if (below_width > select_width) {
 			below_width = select_width;
@@ -1020,7 +1058,8 @@ NetNet* NetESelect::synthesize(Design *des, NetScope*scope, NetExpr*root)
 		  } else {
 			select_width = sub->vector_width() - base_val;
 		  }
-		  unsigned above_width = expr_width() - select_width;
+		  ivl_assert(*this, expr_width() > (select_width+below_width));
+		  unsigned above_width = expr_width() - select_width - below_width;
 
 		  above = make_const_x(des, scope, above_width);
 		  above->set_line(*this);
@@ -1033,11 +1072,13 @@ NetNet* NetESelect::synthesize(Design *des, NetScope*scope, NetExpr*root)
 						  NetPartSelect::VP);
 	    des->add_node(sel);
 
+	    ivl_assert(*this, select_width > 0);
+	    netvector_t*tmp_vec = new netvector_t(sub->data_type(),
+						  select_width-1, 0);
 	    NetNet*tmp = new NetNet(scope, scope->local_symbol(),
-				    NetNet::WIRE, select_width);
-	    tmp->data_type(sub->data_type());
-	    tmp->local_flag(true);
+				    NetNet::WIRE, tmp_vec);
 	    tmp->set_line(*this);
+	    tmp->local_flag(true);
 	    connect(sel->pin(0), tmp->pin(0));
 
 	    unsigned concat_count = 1;
@@ -1060,11 +1101,11 @@ NetNet* NetESelect::synthesize(Design *des, NetScope*scope, NetExpr*root)
 			connect(cat->pin(concat_count), above->pin(0));
 		  }
 
+		  tmp_vec = new netvector_t(sub->data_type(), expr_width()-1, 0);
 		  tmp = new NetNet(scope, scope->local_symbol(),
-				   NetNet::WIRE, expr_width());
-		  tmp->data_type(sub->data_type());
-		  tmp->local_flag(true);
+				   NetNet::WIRE, tmp_vec);
 		  tmp->set_line(*this);
+		  tmp->local_flag(true);
 		  connect(cat->pin(0), tmp->pin(0));
 	    }
 	    return tmp;
@@ -1074,15 +1115,17 @@ NetNet* NetESelect::synthesize(Design *des, NetScope*scope, NetExpr*root)
 	// actual part/bit select. Generate a NetPartSelect object to
 	// do the work, and replace "sub" with the selected output.
       if (base_ != 0) {
-	    off = base_->synthesize(des, scope, root);
+	    NetNet*off = base_->synthesize(des, scope, root);
 
-	    NetPartSelect*sel = new NetPartSelect(sub, off, expr_width());
+	    NetPartSelect*sel = new NetPartSelect(sub, off, expr_width(),
+	                                          base_->has_sign());
 	    sel->set_line(*this);
 	    des->add_node(sel);
 
+	    netvector_t*tmp_vec = new netvector_t(sub->data_type(),
+						  expr_width()-1, 0);
 	    NetNet*tmp = new NetNet(scope, scope->local_symbol(),
-				    NetNet::IMPLICIT, expr_width());
-	    tmp->data_type(sub->data_type());
+				    NetNet::IMPLICIT, tmp_vec);
 	    tmp->local_flag(true);
 	    tmp->set_line(*this);
 	    sub = tmp;
@@ -1098,17 +1141,29 @@ NetNet* NetESelect::synthesize(Design *des, NetScope*scope, NetExpr*root)
       if (sub->vector_width() == expr_width())
 	    return sub;
 
+      netvector_t*net_vec = new netvector_t(expr_type(), expr_width()-1, 0);
+      net_vec->set_signed(has_sign());
+      NetNet*net = new NetNet(scope, scope->local_symbol(),
+			      NetNet::IMPLICIT, net_vec);
+      net->set_line(*this);
+      net->local_flag(true);
+
+	// It may still happen that the expression is wider than the selection,
+	// and there was no part select created earlier (size casting).
+      if(expr_width() < sub->vector_width()) {
+	    NetPartSelect*sel = new NetPartSelect(sub, 0, expr_width(),
+                                                  NetPartSelect::VP, has_sign());
+	    sel->set_line(*this);
+	    des->add_node(sel);
+
+	    connect(net->pin(0), sel->pin(0));
+
 	// The vector_width is not exactly right, so the source is
 	// probably asking for padding. Create nodes to do sign
 	// extension or 0 extension, depending on the has_sign() mode
 	// of the expression.
 
-      NetNet*net = new NetNet(scope, scope->local_symbol(),
-			      NetNet::IMPLICIT, expr_width());
-      net->data_type(expr_type());
-      net->local_flag(true);
-      net->set_line(*this);
-      if (has_sign()) {
+      } else if (has_sign()) {
 	    NetSignExtend*pad = new NetSignExtend(scope,
 						  scope->local_symbol(),
 						  expr_width());
@@ -1117,7 +1172,6 @@ NetNet* NetESelect::synthesize(Design *des, NetScope*scope, NetExpr*root)
 
 	    connect(pad->pin(1), sub->pin(0));
 	    connect(pad->pin(0), net->pin(0));
-	    net->set_signed(true);
 
       } else {
 
@@ -1126,7 +1180,6 @@ NetNet* NetESelect::synthesize(Design *des, NetScope*scope, NetExpr*root)
 	    cat->set_line(*this);
 	    des->add_node(cat);
 
-	    assert(expr_width() > sub->vector_width());
 	    unsigned pad_width = expr_width() - sub->vector_width();
 	    verinum pad((uint64_t)0, pad_width);
 	    NetConst*con = new NetConst(scope, scope->local_symbol(),
@@ -1134,11 +1187,11 @@ NetNet* NetESelect::synthesize(Design *des, NetScope*scope, NetExpr*root)
 	    con->set_line(*this);
 	    des->add_node(con);
 
+	    netvector_t*tmp_vec = new netvector_t(expr_type(), pad_width-1, 0);
 	    NetNet*tmp = new NetNet(scope, scope->local_symbol(),
-				    NetNet::IMPLICIT, pad_width);
-	    tmp->data_type(expr_type());
-	    tmp->local_flag(true);
+				    NetNet::IMPLICIT, tmp_vec);
 	    tmp->set_line(*this);
+	    tmp->local_flag(true);
 	    connect(tmp->pin(0), con->pin(0));
 
 	    connect(cat->pin(0), net->pin(0));
@@ -1189,8 +1242,9 @@ NetNet* NetETernary::synthesize(Design *des, NetScope*scope, NetExpr*root)
       ivl_assert(*this, csig->vector_width() == 1);
 
       unsigned width=expr_width();
-      NetNet*osig = new NetNet(csig->scope(), path, NetNet::IMPLICIT, width);
-      osig->data_type(expr_type());
+      netvector_t*osig_vec = new netvector_t(expr_type(), width-1, 0);
+      NetNet*osig = new NetNet(csig->scope(), path, NetNet::IMPLICIT, osig_vec);
+      osig->set_line(*this);
       osig->local_flag(true);
 
 	/* Make sure the types match. */
@@ -1212,6 +1266,7 @@ NetNet* NetETernary::synthesize(Design *des, NetScope*scope, NetExpr*root)
       perm_string oname = csig->scope()->local_symbol();
       NetMux *mux = new NetMux(csig->scope(), oname, width,
 			       2, csig->vector_width());
+      mux->set_line(*this);
       connect(tsig->pin(0), mux->pin_Data(1));
       connect(fsig->pin(0), mux->pin_Data(0));
       connect(osig->pin(0), mux->pin_Result());
@@ -1229,14 +1284,33 @@ NetNet* NetETernary::synthesize(Design *des, NetScope*scope, NetExpr*root)
  */
 NetNet* NetESignal::synthesize(Design*des, NetScope*scope, NetExpr*root)
 {
+	// If this is a synthesis with a specific value for the
+	// signal, then replace it (here) with a constant value.
+      if (net_->scope()==scope && net_->name()==scope->genvar_tmp) {
+	    netvector_t*tmp_vec = new netvector_t(net_->data_type(),
+						  net_->vector_width()-1, 0);
+	    NetNet*tmp = new NetNet(scope, scope->local_symbol(),
+				    NetNet::IMPLICIT, tmp_vec);
+	    tmp->set_line(*this);
+	    tmp->local_flag(true);
+	    verinum tmp_val ((uint64_t)scope->genvar_tmp_val, net_->vector_width());
+	    NetConst*tmp_const = new NetConst(scope, scope->local_symbol(), tmp_val);
+	    tmp_const->set_line(*this);
+	    des->add_node(tmp_const);
+
+	    connect(tmp->pin(0), tmp_const->pin(0));
+	    return tmp;
+      }
+
       if (word_ == 0)
 	    return net_;
 
+      netvector_t*tmp_vec = new netvector_t(net_->data_type(),
+					    net_->vector_width()-1, 0);
       NetNet*tmp = new NetNet(scope, scope->local_symbol(),
-			      NetNet::IMPLICIT, net_->vector_width());
+			      NetNet::IMPLICIT, tmp_vec);
       tmp->set_line(*this);
       tmp->local_flag(true);
-      tmp->data_type(net_->data_type());
 
 	// For NetExpr objects, the word index is already converted to
 	// a canonical (lsb==0) address. Just use the index directly.
@@ -1267,15 +1341,16 @@ static NetEvWait* make_func_trigger(Design*des, NetScope*scope, NetExpr*root)
       NetEvWait*trigger = 0;
 
       NexusSet*nset = root->nex_input(false);
-      if (nset && (nset->count() > 0)) {
+      if (nset && (nset->size() > 0)) {
             NetEvent*ev = new NetEvent(scope->local_symbol());
             ev->set_line(*root);
 
             NetEvProbe*pr = new NetEvProbe(scope, scope->local_symbol(),
                                            ev, NetEvProbe::ANYEDGE,
-                                           nset->count());
-            for (unsigned idx = 0 ;  idx < nset->count() ;  idx += 1)
-                  connect(nset[0][idx], pr->pin(idx));
+                                           nset->size());
+            pr->set_line(*root);
+            for (unsigned idx = 0 ;  idx < nset->size() ;  idx += 1)
+                  connect(nset->at(idx).lnk, pr->pin(idx));
 
             des->add_node(pr);
 
@@ -1288,7 +1363,7 @@ static NetEvWait* make_func_trigger(Design*des, NetScope*scope, NetExpr*root)
       delete nset;
 
       return trigger;
-}  
+}
 
 NetNet* NetESFunc::synthesize(Design*des, NetScope*scope, NetExpr*root)
 {
@@ -1311,26 +1386,26 @@ NetNet* NetESFunc::synthesize(Design*des, NetScope*scope, NetExpr*root)
       }
 
       NetEvWait*trigger = 0;
-      if (nparms_ == 0) {
+      if (parms_.empty()) {
             trigger = make_func_trigger(des, scope, root);
       }
 
       NetSysFunc*net = new NetSysFunc(scope, scope->local_symbol(),
-				      def, 1+nparms_, trigger);
+				      def, 1+parms_.size(), trigger);
       net->set_line(*this);
       des->add_node(net);
 
+      netvector_t*osig_vec = new netvector_t(def->type, def->wid-1, 0);
+      osig_vec->set_signed(def->type==IVL_VT_REAL? true : false);
       NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::WIRE, def->wid);
-      osig->local_flag(true);
-      osig->set_signed(def->type==IVL_VT_REAL? true : false);
-      osig->data_type(def->type);
+			       NetNet::WIRE, osig_vec);
       osig->set_line(*this);
+      osig->local_flag(true);
 
       connect(net->pin(0), osig->pin(0));
 
       unsigned errors = 0;
-      for (unsigned idx = 0 ;  idx < nparms_ ;  idx += 1) {
+      for (unsigned idx = 0 ;  idx < parms_.size() ;  idx += 1) {
 	    NetNet*tmp = parms_[idx]->synthesize(des, scope, root);
 	    if (tmp == 0) {
 		  cerr << get_fileline() << ": error: Unable to elaborate "
@@ -1351,11 +1426,15 @@ NetNet* NetESFunc::synthesize(Design*des, NetScope*scope, NetExpr*root)
 
 NetNet* NetEUFunc::synthesize(Design*des, NetScope*scope, NetExpr*root)
 {
-      svector<NetNet*> eparms (parms_.count());
+      vector<NetNet*> eparms (parms_.size());
 
         /* Synthesize the arguments. */
       bool errors = false;
-      for (unsigned idx = 0; idx < eparms.count(); idx += 1) {
+      for (unsigned idx = 0; idx < eparms.size(); idx += 1) {
+	    if (dynamic_cast<NetEEvent*> (parms_[idx])) {
+		  errors = true;
+		  continue;
+	    }
 	    NetNet*tmp = parms_[idx]->synthesize(des, scope, root);
 	    if (tmp == 0) {
 		  cerr << get_fileline() << ": error: Unable to synthesize "
@@ -1384,18 +1463,32 @@ NetNet* NetEUFunc::synthesize(Design*des, NetScope*scope, NetExpr*root)
       des->add_node(net);
 
         /* Create an output signal and connect it to the function. */
+      netvector_t*osig_vec = new netvector_t(result_sig_->expr_type(),
+					     result_sig_->vector_width()-1, 0);
       NetNet*osig = new NetNet(scope_, scope_->local_symbol(), NetNet::WIRE,
-                               result_sig_->vector_width());
+                               osig_vec);
+      osig->set_line(*this);
       osig->local_flag(true);
-      osig->data_type(result_sig_->expr_type());
       connect(net->pin(0), osig->pin(0));
 
+      if (debug_synth2) {
+	    cerr << get_fileline() << ": NetEUFunc::synthesize: "
+		 << "result_sig_->vector_width()=" << result_sig_->vector_width()
+		 << ", osig->vector_width()=" << osig->vector_width() << endl;
+      }
+
         /* Connect the pins to the arguments. */
       NetFuncDef*def = func_->func_def();
-      for (unsigned idx = 0; idx < eparms.count(); idx += 1) {
-	    NetNet*tmp = pad_to_width(des, eparms[idx],
-	                              def->port(idx)->vector_width(), *this);
-	    connect(net->pin(idx+1), tmp->pin(0));
+      for (unsigned idx = 0; idx < eparms.size(); idx += 1) {
+	    unsigned width = def->port(idx)->vector_width();
+	    NetNet*tmp;
+	    if (eparms[idx]->get_signed()) {
+		  tmp = pad_to_width_signed(des, eparms[idx], width, *this);
+	    } else {
+		  tmp = pad_to_width(des, eparms[idx], width, *this);
+	    }
+	    NetNet*tmpc = crop_to_width(des, tmp, width);
+	    connect(net->pin(idx+1), tmpc->pin(0));
       }
 
       return osig;
